"""
data_collector.py

Main Module

Description: Module for collecting data from an ontology endpoint and exploring ontology data.

Author: Sadegh Abadijou
Email: s.abadijou@gmail.com
Created: 22/1/2024
"""

from .an_ontology import AnOntology
from .config import endpoint_info
import requests
import logging
import os


class DataCollector:
    def __init__(self, ontology_endpoint, **kwargs):
        """
        :param ontology_endpoint: the base URI for the ontology endpoint
        :param kwargs: Arbitrary keyword arguments, typically includes 'ontology_id'.
        """
        self.uri = ontology_endpoint

        self.kwargs = kwargs

    def fetch_data(self):
        """
        Fetch data from the ontology endpoint. Handle HTTP and general errors.
        :return: The JSON response data from the ontology endpoint or None if an error occurs.
        """

        try:
            response = requests.get(os.path.join(self.uri, self.kwargs.get('ontology_id')))
            response.raise_for_status()
            return response.json()
        except requests.HTTPError as http_error:
            # ToDo The api return 500 instead of 404 when ontology not exists
            if http_error.response.status_code == 500:
                logging.error(f'No content found for ontology ID')
            elif http_error.response.status_code == 503:
                logging.error('Service unavailable')
            return None
        except Exception as error:
            logging.error(f'Other error occurred: {error}')
            return error


def explore_ontology(ontology_id, **kwargs):
    """
    Explore and return an instance of AnOntology based on the provided ontology_id and other parameters.

    :param ontology_id: The ID of the ontology to be fetched and explored.
    :param kwargs: Arbitrary keyword arguments used for additional arguments.
    :return: An instance of specific Ontology
    """
    collector = DataCollector(ontology_endpoint=endpoint_info["uri"],
                              ontology_id=ontology_id)
    data = collector.fetch_data()
    if data is not None:
        ontology_instance = AnOntology(**data)
        return ontology_instance
    else:
        print("Failed to retrieve data.")




mock_response = {
  "languages" : [ "cs", "dtp", "en", "es", "fr", "ja", "nl", "nna", "tr", "tw", "zh" ],
  "lang" : "en",
  "ontologyId" : "hp",
  "loaded" : "2024-01-19T16:56:57.494460839",
  "updated" : "2024-01-19T16:56:57.494460839",
  "status" : "LOADED",
  "message" : "",
  "version" : "2024-01-16",
  "fileHash" : "null",
  "loadAttempts" : 0,
  "numberOfTerms" : 30297,
  "numberOfProperties" : 532,
  "numberOfIndividuals" : 0,
  "config" : {
    "id" : "hp",
    "versionIri" : "http://purl.obolibrary.org/obo/hp/releases/2024-01-16/hp-international.owl",
    "namespace" : "hp",
    "preferredPrefix" : "HP",
    "title" : "Human Phenotype Ontology",
    "description" : "The Human Phenotype Ontology (HPO) provides a standardized vocabulary of phenotypic abnormalities and clinical features encountered in human disease.",
    "homepage" : "http://www.human-phenotype-ontology.org/",
    "version" : "2024-01-16",
    "mailingList" : "https://groups.io/g/human-phenotype-ontology",
    "tracker" : "https://github.com/obophenotype/human-phenotype-ontology/issues/",
    "logo" : "null",
    "creators" : [ ],
    "annotations" : "null",
    "fileLocation" : "http://purl.obolibrary.org/obo/hp/hp-international.owl",
    "oboSlims" : "false",
    "labelProperty" : "http://www.w3.org/2000/01/rdf-schema#label",
    "definitionProperties" : [ ],
    "synonymProperties" : [ ],
    "hierarchicalProperties" : [ ],
    "baseUris" : [ ],
    "hiddenProperties" : [ ],
    "preferredRootTerms" : [ ],
    "isSkos" : "false",
    "allowDownload" : "false"
  },
  "baseUris" : [ ],
  "_links" : {
    "self" : {
      "href" : "http://www.ebi.ac.uk/ols4/api/ontologies/hp?lang=en"
    },
    "terms" : {
      "href" : "http://www.ebi.ac.uk/ols4/api/ontologies/hp/terms"
    },
    "properties" : {
      "href" : "http://www.ebi.ac.uk/ols4/api/ontologies/hp/properties"
    },
    "individuals" : {
      "href" : "http://www.ebi.ac.uk/ols4/api/ontologies/hp/individuals"
    }
  }
}


from srse_manchester.srse_manchester import DataCollector, explore_ontology
from unittest.mock import patch, Mock, mock_open
from .mock_response import mock_response as MR
import requests
import pytest


@pytest.fixture
def mock_response():
    mock = Mock(spec=requests.Response)
    mock.json.return_value = MR
    mock.status_code = 200
    return mock


@pytest.fixture
def mock_config():
    return {"api": {"ontology_uri": "http://www.ebi.ac.uk/ols4/api/ontologies/"}}


def test_data_collector_initialization():
    """
    Test : Ensures that the DataCollector instance is correctly initialized with a specified URI and ontology_id.
    """
    collector = DataCollector("http://www.ebi.ac.uk/ols4/api/ontologies/", ontology_id="hp")
    assert collector.uri == "http://www.ebi.ac.uk/ols4/api/ontologies/"
    assert collector.kwargs.get('ontology_id') == "hp"


@patch('requests.get')
def test_fetch_data_valid(mock_get, mock_response):
    """
    Test : fetch_data function in DataCollector with a valid response.
    """
    mock_get.return_value = mock_response
    collector = DataCollector("http://www.ebi.ac.uk/ols4/api/ontologies/", ontology_id="hp")
    response = collector.fetch_data()
    assert response == MR


@patch('builtins.open', new_callable=mock_open,
       read_data="api: {ontology_uri: 'http://www.ebi.ac.uk/ols4/api/ontologies/'}")
@patch('requests.get')
def test_explore_ontology_valid(mock_get, mock_file, mock_response, mock_config):
    """
    Test : explore_ontology function with valid parameters.
    """
    mock_get.return_value = mock_response
    ontology = explore_ontology("hp", **mock_config)
    assert ontology is not None


@patch('requests.get')
def test_fetch_data_service_unavailable(mock_get):
    """
    Test : fetch_data : service unavailable
    """
    mock_get.side_effect = requests.HTTPError(response=Mock(status_code=503))

    collector = DataCollector("http://www.ebi.ac.uk/ols4/api/ontologies123/", ontology_id="unavailable_service")
    result = collector.fetch_data()

    assert result is None


@patch('requests.get')
def test_fetch_data_http_error(mock_get):
    """
    Test : fetch_data_http_error
    """
    mock_get.side_effect = requests.HTTPError(response=Mock(status_code=500))

    collector = DataCollector("http://www.ebi.ac.uk/ols4/api/ontologies/", ontology_id="invalid_ontology")
    result = collector.fetch_data()

    assert result is None





"""
an_ontology.py

Description: Module to handle specific ontology operations.

Author: Sadegh Abadijou
Email: s.abadijou@gmail.com
Created: 22/1/2024

"""

from .ontology_abstract import Ontology
from abc import ABC


class AnOntology(Ontology, ABC):
    """
    Extend Ontology
    Represents a specific ontology with print functionality.
    """
    def __init__(self, **kwargs):
        """
        :param kwargs: Keyword arguments for Ontology parameters.
        """
        super().__init__(**kwargs)

    def print_content(self):
        """
        Overwrite print_content from abstract Ontology class
        Print : Ontology title
        Print : Ontology description
        Print : How many terms Ontology includes
        Print : Current status of Ontology
        :return:
        """
        print('Ontology title : {ontology_title} \n '
              'Ontology description : {ontology_description} \n '
              'Number of terms : {number_of_terms} \n '
              'Current status : {current_status} '.format(ontology_title=self.config.title,
                                                          ontology_description=self.config.description,
                                                          number_of_terms=self.number_of_terms,
                                                          current_status=self.status))



from setuptools import setup, find_packages

setup(
    name='srse_manchester',
    version='1.0.1',
    packages=find_packages(),
    description='My Ontology Lookup Service',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    url='https://github.com/sabadijou/srse_manchester/',
    author='Sadegh Abadijou',
    author_email='s.abadijou@gmail.com',
    license='MIT',
    include_package_data=True,
    install_requires=[
        'certifi==2023.11.17',
        'charset-normalizer==3.3.2',
        'colorama==0.4.6',
        'coverage==7.4.0',
        'exceptiongroup==1.2.0',
        'idna==3.6',
        'iniconfig==2.0.0',
        'packaging==23.2',
        'pluggy==1.3.0',
        'pytest==7.4.4',
        'pytest-cov==4.1.0',
        'requests==2.31.0',
        'tomli==2.0.1',
        'urllib3==2.1.0'
    ],
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
    ],
)



"""
ontology_abstract.py

Description: Module to handle ontology abstract class.

Author: Sadegh Abadijou
Email: s.abadijou@gmail.com
Created: 22/1/2024

"""

from abc import abstractmethod, ABC


class Ontology(ABC):
    """
    Base module representing ontology abstract class.
    """
    def __init__(self, **kwargs):
        """
        :param kwargs: Keyword arguments for Ontology parameters.
        """
        self.languages = kwargs.get('languages', None)

        self.lang = kwargs.get('lang', None)

        self.ontology_id = kwargs.get('ontologyId', None)

        self.loaded = kwargs.get('loaded', None)

        self.updated = kwargs.get('updated', None)

        self.status = kwargs.get('status', None)

        self.message = kwargs.get('message', None)

        self.version = kwargs.get('version', None)

        self.fileHash = kwargs.get('fileHash', None)

        self.loadAttempts = kwargs.get('loadAttempts', None)

        self.number_of_terms = kwargs.get('numberOfTerms', None)

        self.number_of_properties = kwargs.get('numberOfProperties', None)

        self.number_of_individuals = kwargs.get('numberOfIndividuals', None)

        self.config = Config(**kwargs.get('config', {}))

        self.base_uris = kwargs.get('baseUris', None)

        self.links = kwargs.get('_links', None)

    @abstractmethod
    def print_content(self):
        pass


class Config:
    """
    Config class for additional ontology metadata.
    """
    def __init__(self, **kwargs):
        """
        :param kwargs: Keyword arguments for Config parameters of Ontology.
        """
        self.versionIri = kwargs.get('versionIri', None)

        self.namespace = kwargs.get('namespace', None)

        self.preferredPrefix = kwargs.get('preferredPrefix', None)

        self.title = kwargs.get('title', None)

        self.description = kwargs.get('description', None)

        self.homepage = kwargs.get('homepage', None)

        self.version = kwargs.get('version', None)

        self.mailingList = kwargs.get('mailingList', None)

        self.tracker = kwargs.get('tracker', None)

        self.logo = kwargs.get('logo', None)

        self.creators = kwargs.get('creators', None)

        self.annotations = kwargs.get('annotations', None)

        self.fileLocation = kwargs.get('fileLocation', None)

        self.oboSlims = kwargs.get('oboSlims', None)

        self.labelProperty = kwargs.get('labelProperty', None)

        self.definitionProperties = kwargs.get('definitionProperties', None)

        self.synonymProperties = kwargs.get('synonymProperties', None)

        self.hierarchicalProperties = kwargs.get('hierarchicalProperties', None)

        self.baseUris = kwargs.get('baseUris', None)

        self.hiddenProperties = kwargs.get('hiddenProperties', None)

        self.preferredRootTerms = kwargs.get('preferredRootTerms', None)

        self.isSkos = kwargs.get('isSkos', None)

        self.allowDownload = kwargs.get('allowDownload', None)

